{-# LANGUAGE OverloadedStrings #-}

module Network.Wai.Logger.Format (apacheFormat) where

import Data.ByteString.Char8 ()
import Data.CaseInsensitive
import Data.Maybe
import Network.HTTP.Types
import Network.Wai
import Network.Wai.Logger.Date
import Network.Wai.Logger.IO
import Network.Wai.Logger.Utils

-- | Apache style log format.
apacheFormat :: ZonedDate -> Request -> Status -> Maybe Integer -> [LogStr]
apacheFormat tmstr req st msize = [
    LS addr
  , LB " - - ["
  , LB tmstr
  , LB "] \""
  , LB $ requestMethod req
  , LB " "
  , LB $ rawPathInfo req
  , LB " "
  , LS $ show . httpVersion $ req
  , LB "\" "
  , LS . show . statusCode $ st
  , LB " "
  , LS $ maybe "-" show msize
  , LB " \""
  , LB $ lookupRequestField' "referer" req
  , LB "\" \""
  , LB $ lookupRequestField' "user-agent" req
  , LB "\"\n"
  ]
  where
    addr = showSockAddr (remoteHost req)

lookupRequestField' :: CI Ascii -> Request -> Ascii
lookupRequestField' k req = fromMaybe "" . lookup k $ requestHeaders req
